#pragma once
/* -copyright-
*/
#include "../src/Pixmaps/flake0.xpm"
#include "../src/Pixmaps/flake1.xpm"
#include "../src/Pixmaps/flake2.xpm"
#include "../src/Pixmaps/flake3.xpm"
#include "../src/Pixmaps/flake4.xpm"
#include "../src/Pixmaps/flake5.xpm"
#include "../src/Pixmaps/flake6.xpm"
#define SNOW_ALL \
SNOW(0) \
SNOW(1) \
SNOW(2) \
SNOW(3) \
SNOW(4) \
SNOW(5) \
SNOW(6) \

